
#include <ntddk.h>
#include <strsafe.h>

#ifdef __cplusplus
extern "C"
{
#endif
    NTSTATUS MajorDeviceControl(PDEVICE_OBJECT pDeviceObject, PIRP pIrp);
    NTSTATUS MajorCreate(IN PDEVICE_OBJECT pDeviceObject, IN PIRP pIrp);
    NTSTATUS MajorClose(IN PDEVICE_OBJECT pDeviceObject, IN PIRP pIrp);
    VOID DriverUnload( PDRIVER_OBJECT pDriverObject );
    NTSTATUS DriverEntry(PDRIVER_OBJECT pDriverObject, PUNICODE_STRING pRegistryPath);
#ifdef __cplusplus
}
#endif

#include "PortIo.h"

#if defined( TARGET_PLATFORM_VER )
#if defined(VER_PRODUCTBUILD)
#undef VER_PRODUCTBUILD 
#define VER_PRODUCTBUILD TARGET_PLATFORM_VER
#else
#define VER_PRODUCTBUILD TARGET_PLATFORM_VER
#endif
#endif

VOID DriverUnload( PDRIVER_OBJECT pDriverObject )
{
    UNICODE_STRING uniDOSString;

    RtlInitUnicodeString(&uniDOSString, DOS_DEVICE_NAME);
    IoDeleteSymbolicLink (&uniDOSString);
    IoDeleteDevice(pDriverObject->DeviceObject);

    DbgPrint("Driver Unload\n" );
}

NTSTATUS DriverEntry(PDRIVER_OBJECT pDriverObject, PUNICODE_STRING pRegistryPath)
{
    NTSTATUS ntStatus;
    PDEVICE_OBJECT DeviceObject = NULL;
    UNICODE_STRING uniNameString, uniDOSString;

    RtlInitUnicodeString(&uniNameString, DEVICE_NAME);
    RtlInitUnicodeString(&uniDOSString, DOS_DEVICE_NAME);

    ntStatus = IoCreateDevice(pDriverObject, 
        0,
        &uniNameString,
        FILE_DEVICE_UNKNOWN,
        0, 
        FALSE, 
        &DeviceObject);

    if (NT_SUCCESS(ntStatus) == FALSE)
    {
        return ntStatus;
    }

    ntStatus = IoCreateSymbolicLink (&uniDOSString, &uniNameString);

    if (NT_SUCCESS(ntStatus) == FALSE)
    {
        return ntStatus;
    }

    pDriverObject->MajorFunction[IRP_MJ_CREATE]         = MajorCreate;
    pDriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = MajorDeviceControl;
    pDriverObject->MajorFunction[IRP_MJ_CLOSE]          = MajorClose;
    pDriverObject->DriverUnload                         = DriverUnload;

    return STATUS_SUCCESS;
}

NTSTATUS MajorDeviceControl(PDEVICE_OBJECT pDeviceObject, PIRP pIrp)
{
    PIO_STACK_LOCATION  irpSp;
    NTSTATUS            ntStatus = STATUS_SUCCESS;   

    ULONG               inBufLength;   /* Input buffer length */
    ULONG               outBufLength;  /* Output buffer length */

    PUCHAR              CharBuffer; 
    PUSHORT             ShortBuffer;
    PULONG              LongBuffer;
    PVOID               ioBuffer;

    UCHAR Value;

    irpSp = IoGetCurrentIrpStackLocation( pIrp );
    inBufLength = irpSp->Parameters.DeviceIoControl.InputBufferLength;
    outBufLength = irpSp->Parameters.DeviceIoControl.OutputBufferLength;

    ioBuffer    = pIrp->AssociatedIrp.SystemBuffer;

    CharBuffer  = (PUCHAR) ioBuffer;
    ShortBuffer = (PUSHORT) ioBuffer;
    LongBuffer  = (PULONG) ioBuffer;

    USHORT processId[1] = { 0x61 };

    switch ( irpSp->Parameters.DeviceIoControl.IoControlCode )
    {
    case IOCTL_WRITE_PROCESSID:
        if (inBufLength >= 4) 
        {
            DbgPrint("Write - 0x%x\n", LongBuffer[0]);
             
            HANDLE ProcessHandle = NULL;
            OBJECT_ATTRIBUTES ObjectAttributes;
            CLIENT_ID ClientId;

            ObjectAttributes.Length = sizeof(OBJECT_ATTRIBUTES);
            ObjectAttributes.RootDirectory = NULL;
            ObjectAttributes.ObjectName = NULL;
            ObjectAttributes.Attributes = OBJ_KERNEL_HANDLE | OBJ_CASE_INSENSITIVE;
            ObjectAttributes.SecurityDescriptor = NULL;
            ObjectAttributes.SecurityQualityOfService = NULL;

            ClientId.UniqueThread = 0;
            ClientId.UniqueProcess = (HANDLE)LongBuffer[0];

            NTSTATUS NtStatus = ZwOpenProcess(
                &ProcessHandle,
                PROCESS_ALL_ACCESS,
                &ObjectAttributes,
                &ClientId);

            if (NtStatus == STATUS_SUCCESS)
            {
                NtStatus = ZwTerminateProcess(
                    ProcessHandle,
                    STATUS_SUCCESS);

                if (NtStatus)
                    KdPrint((" [!] Could not to terminate process...\n"));
            }
        } 
        else 
        {
            ntStatus = STATUS_BUFFER_TOO_SMALL;
        }

        pIrp->IoStatus.Information = 0; /* Output Buffer Size */
        ntStatus = STATUS_SUCCESS;
        break;

    default:
        ntStatus = STATUS_UNSUCCESSFUL;
        pIrp->IoStatus.Information = 0;
        break;
    }

    pIrp->IoStatus.Status = ntStatus;
    IoCompleteRequest( pIrp, IO_NO_INCREMENT );
    return ntStatus;
}

NTSTATUS MajorCreate(IN PDEVICE_OBJECT pDeviceObject, IN PIRP pIrp)
{
    DbgPrint("MajorCreate\n");
    NTSTATUS ntStatus;

    pIrp->IoStatus.Information = 0;
    pIrp->IoStatus.Status = STATUS_SUCCESS;

    ntStatus = pIrp->IoStatus.Status;
    IoCompleteRequest(pIrp, IO_NO_INCREMENT);

    return STATUS_SUCCESS;
}

NTSTATUS MajorClose(IN PDEVICE_OBJECT pDeviceObject, IN PIRP pIrp)
{
    DbgPrint("MajorClose\n");
    NTSTATUS ntStatus;

    pIrp->IoStatus.Status = STATUS_SUCCESS;
    pIrp->IoStatus.Information = 0;

    ntStatus = pIrp->IoStatus.Status;
    IoCompleteRequest(pIrp, IO_NO_INCREMENT);

    return ntStatus;
}